// This is -*- C++ -*-
// $Id: Access.h,v 1.2 1998/12/30 02:21:39 trow Exp $

/* 
 * Access.h
 *
 * Copyright (C) 1998 Asger Alstrup Nielsen
 *
 * Developed by Asger Alstrup Nielsen <alstrup@diku.dk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_ACCESS_H
#define _INC_ACCESS_H

#include "Exception.h"
#include "Value.h"
#include "StringSet.h"

// A class to keep easy access methods for our polymorphic classes
// Using this, we avoid cluttering the code with casts.
// (A namespace would be perfect for this.)
struct Access {
  static string get_string(Value const & v) { 
    if (v.type() != 0 && v.type()->kind() != DataType::String) { 
      throw Exception("Type error: Requires string value.");
    }
    return ((String const &)(v)).get_string();
  }
  static double get_real(Value const & v) {
    if (v.type() != 0 && v.type()->kind() != DataType::Real) { 
      throw Exception("Type error: Requires real value.");
    }
    return ((Real const &)(v)).get_real();
  }

  static StringSet::representation const & strings(DataSet const & s) {
    return ((StringSet const &)(s)).strings();
  }
  static StringSet::representation & strings(DataSet & s) {
    return ((StringSet &)(s)).strings();
  }
};

#endif
