// This is -*- C++ -*-
// $Id: CategoricalSet.cpp,v 1.2 1998/12/06 03:02:56 trow Exp $

/* 
 * CategoricalSet.cpp
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include <config.h>
#include "CategoricalSet.h"

CategoricalSet::CategoricalSet(const Category& c) :
  cat_(c), label_("Unlabelled")
{
  freq_init();
}

CategoricalSet::CategoricalSet(catkey_t k) : 
  cat_(k), label_("Unlabelled")
{
  freq_init();
}

CategoricalSet::CategoricalSet(const string& l) : 
  cat_(l), label_("Unlabelled")
{
  freq_init();
}

CategoricalSet::CategoricalSet(const CategoricalSet& cs) :
  cat_(cs.category()), data_(cs.data_), freq_(cs.freq_)
{
  label_ = "Copy of " + cs.label();
}

CategoricalSet::~CategoricalSet()
{

}

CategoricalSet&
CategoricalSet::operator=(const CategoricalSet& cs)
{
  if (this != &cs) {
    cat_ = cs.category();
    data_ = cs.data_;
    freq_ = cs.freq_;
    label_ = "Copy of " + cs.label();
  }

  return *this;
}

void
CategoricalSet::reserve(size_t R)
{
  data_.reserve(R);
}

void
CategoricalSet::add(catcode_t c)
{
  data_.add(c);
  ++(freq_.data()[c]);
  
}

void
CategoricalSet::clear()
{
  data_.clear();
  freq_init();
}

void
CategoricalSet::reverse()
{
  catcode_t* c = data_.data();
  for(size_t i=0; i<size()/2; ++i) {
    catcode_t x = c[i];
    c[i] = c[size()-i-1];
    c[size()-i-1] = x;
  }
}

void
CategoricalSet::sort(int dir)
{
  // Since we count frequencies, we can avoid actually doing any sort of 
  // a real sort.
  catcode_t* c = data_.data();
  int ptr=-1;
  size_t N=data_.size(), remaining=0;

  size_t i=0, j=N-1;
  size_t& k = dir >= 0 ? i : j;
  for(; i<N; ++i, --j) {
    while (remaining == 0) {
      ++ptr;
      assert(ptr < (int)freq_.size());
      remaining = (freq_.data())[ptr];
    }
    c[k] = ptr;
    --remaining;
  }
}

void
CategoricalSet::scramble()
{
  if (size() < 2) return;

  catcode_t* c = data_.data();
  size_t N = size();

  for(size_t i=0; i<N; ++i) {
    size_t j = random().random_integer(i,N-1);
      if (i != j) {
      catcode_t x = c[i];
      c[i] = c[j];
      c[j] = x;
    }
  }
}

void
CategoricalSet::permute(const Permutation& p)
{
  if (size() != p.size())
    throw Exception("Permutation and data are not of the same size");

  catcode_t* c = data_.data();
  p.permute(c);
}

Permutation
CategoricalSet::sorting_permutation(int dir) const
{
  Permutation p;

  size_t tally=0;
  vector<size_t> lookup;
  for(size_t i=0; i<cat_.size(); ++i) {
    lookup.push_back(tally);
    tally += frequency((catcode_t)i);
  }

  const catcode_t* c = data();
  size_t* map = new size_t[size()];
  for(size_t i=0; i<size(); ++i) {
    map[i] = lookup[c[i]];
    lookup[c[i]]++;
  }

  if (dir < 0) {
    for(size_t i=0; i<size()/2; ++i) {
      size_t x = map[i];
      map[i] = map[size()-1-i];
      map[size()-1-i] = x;
    }
  }

  p.initialize(size(), map);
  delete [] map;

  return p;
}

void
CategoricalSet::freq_init()
{
  freq_.reserve(cat_.size());
  for(size_t i=0; i<cat_.size(); ++i)
    freq_.add(0);
}

// $Id: CategoricalSet.cpp,v 1.2 1998/12/06 03:02:56 trow Exp $
