// This is -*- C++ -*-
// $Id: CategoricalSet.h,v 1.2 1998/12/30 02:21:40 trow Exp $

/* 
 * CategoricalSet.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#ifndef _INC_CATEGORICALSET_H
#define _INC_CATEGORICALSET_H

#include "Category.h"
#include "RCArray.h"
#include "Permutation.h"
#include "Exception.h"
#include "DataSet.h"

class CategoricalSet : public DataSet {
public:
  CategoricalSet(const Category&);
  CategoricalSet(catkey_t);
  CategoricalSet(const string&);
  CategoricalSet(const CategoricalSet&);

  ~CategoricalSet();

  CategoricalSet& operator=(const CategoricalSet&);

  const Category& category() const { return cat_; }

  const string& label() const { return label_; }
  void set_label(const string& s) { label_ = s; }

  size_t size() const { return data_.size(); }

  const catcode_t* data() const { return data_.data(); }
  catcode_t data(size_t i) const { return (data_.data())[i]; }
  const string& item(size_t i) const { return cat_.decode(data(i)); }

  size_t frequency(catcode_t i) const { return (freq_.data())[i]; }
  size_t frequency(const string& s) const { return frequency(cat_.code(s)); }

  void reserve(size_t);

  void add(catcode_t);
  void add(const string& s) { add(cat_.code(s)); }

  void clear();

  void reverse();
  void sort(int dir = 1);
  void scramble();
  void permute(const Permutation&);

  Permutation sorting_permutation(int dir = 1) const;


private:
  void freq_init();

  Category cat_;
  string label_;
  RCArray<catcode_t> data_;
  RCArray<size_t>freq_;
};

#endif // _INC_CATEGORICALSET_H

// $Id: CategoricalSet.h,v 1.2 1998/12/30 02:21:40 trow Exp $
