// This is -*- C++ -*-
// $Id: Category.h,v 1.1 1998/12/06 02:31:57 trow Exp $

/* 
 * Category.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_CATEGORY_H
#define _INC_CATEGORY_H

#include <string>
#include <map>
#include <vector>

#include "Exception.h"

// These should be unsigned 16-bit int.  It can be bigger, but that
// seems excessive... :-)
typedef unsigned short catcode_t; 
typedef unsigned short catkey_t;

/*
  Category is the "handle", Category_Impl is the "body".

  Category_Impl should really be defined inside of Category, but that
  might not work on older compilers...
*/

class Category_Impl {

  friend class Category;

public:
  ~Category_Impl();

private:
  Category_Impl(catkey_t);

  catkey_t category_key() const { return key_; }
  const string& name() const { return name_; }

  void set_name(const string&);
  void add(const string&);
  void make_ordinal();
  void complete_category();

  bool locked() const { return locked_; }
  size_t size() const { return code_lookup_.size(); }
  bool ordinal() const { return ordinal_; }

  catcode_t code(const string&) const;
  const string& decode(catcode_t) const;

private:
  catkey_t key_;
  string name_;

  bool locked_;
  bool ordinal_;
  map<string, catcode_t> string_lookup_;
  vector<string> code_lookup_;
};

class Category {
public:
  Category();
  Category(catkey_t);
  Category(const string&);
  Category(const Category&);

  ~Category();

  Category& operator=(const Category&);

  bool valid() const { return impl_ != 0; }
  bool invalid() const { return !valid(); }

  void assert_valid(catcode_t c) const {
    if (c >= size()) {
      throw Exception("Invalid category code");
    }
  }

  catkey_t category_key() const;
  const string& name() const;

  void set_name(const string&);
  void add(const string&);
  void make_ordinal();
  void complete_category();

  bool locked() const;
  size_t size() const;
  bool ordinal() const;

  catcode_t code(const string&) const;
  const string& decode(catcode_t) const;

private:
  Category_Impl* impl_;

  static catkey_t next_key_;
  static map<catkey_t, Category_Impl*> impl_map_;
  static map<string, Category_Impl*> impl_stringmap_;
};

#endif // _INC_CATEGORY_H

// $Id: Category.h,v 1.1 1998/12/06 02:31:57 trow Exp $
