// This is -*- C++ -*-

/* 
 * DataType.cpp
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Havoc Pennington <hp@pobox.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include <stdio.h>
#include <math.h>
#include <cstring>
#include <cstdio>

#include "DataType.h"
#include "Value.h"
#include "Access.h"

// Implemenation of the DataType ---------------

string DataType::name() const { 
  return "Unknown"; 
}

string DataType::format(Value const & v) const { 
  return "Unknown"; 
}

// Implemenation of the RealType ---------------

string RealType::name() const {
  return "Real";
}

string RealType::format(Value const & d) const { 
  // TODO: Modify to handle setup information
  char buf[100]; 
  sprintf(buf,"%f", Access::get_real(d));
  return buf;
}

Value * RealType::parse(const string & text, string & remains) const {
  // TODO: Modify to handle setup information
  Value * v = 0;
  errno = 0;
  char * upto;
  double d = strtod(text.c_str(), &upto);
  if (errno == 0) {
    v = ValueFactory::factory()->create(d);
  } else {
    // v = 0;
  }
  remains = upto;
  return v;
}

Value * RealType::create_default_value() const {
  return ValueFactory::factory()->create(0.0);
}

double RealType::identify (string const & text) const {
  // TODO: Modify to handle setup information

  // First make a copy to work with
  string s = text;

  size_t p = s.find_first_of("eE",0);

  // if we have an exponent, slice it off and check it
  if (p != s.npos) {
    string expo = s.substr(p);
    s.erase(p);

    if (expo.size() < 2)
      return 0.0;
    
    // make sure that both the 1e10, 1e+10 and 1e-10 type notation
    // is handled correctly
    if (expo[1] == '-' || expo[1] == '+') {
      if (expo.size() < 3)
	return 0.0;
    } else if (!isdigit(expo[1]))
      return 0.0;

    // make sure that the exponent contains only digits
    if (expo.find_first_not_of("0123456789",2) != expo.npos)
      return 0.0;
  }

  // the empty string is not a float
  if (s.size() == 0)
    return 0.0;

  // check the mantissa
  bool seenDecPt = false;
  for(size_t i=0; i<s.size(); ++i) {
    char c = s[i];
    if (!(isdigit(c) ||
	  (c == '-' && i == 0) || // only allow negative in first place
	  (c == '.' && !seenDecPt))) // only one dec pt per mantissa
      return 0.0;
    if (c == '.')
      seenDecPt = true;
  }
  return 1.0;
}

void RealType::autosetup(string_tools::strings const & learning) {
  // TODO: Modify to extract setup information
}

void RealType::assign(DataType const & d) {
  // TODO: Modify to copy setup information
}

// Implemenation of the StringType ---------------

string StringType::name() const {
  return "String";
}

string StringType::format(Value const & v) const {
  // TODO: Add quoting support
  return Access::get_string(v);
}

Value * StringType::parse(string const & v, string & remains) const {
  // TODO: Add quoting support
  return ValueFactory::factory()->create(v);
}

Value * StringType::create_default_value() const {
  return ValueFactory::factory()->create(string());
}

double StringType::identify(string const & v) const {
  // TODO: Add quoting support
  return 1.0;
}

void StringType::autosetup(string_tools::strings const & learning) {
  // TODO: Add quoting detection
}
