// This is -*- C++ -*-
// $Id: StringSet.cpp,v 1.2 1998/12/30 02:21:45 trow Exp $

/* 
 * StringSet.cpp
 *
 * Copyright (C) 1998 Asger Alstrup Nielsen
 *
 * Developed by Asger Alstrup Nielsen <alstrup@diku.dk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include "StringSet.h"
#include "Access.h"
#include "Permutation.h"

StringSet& StringSet::operator=(StringSet const & s) {
  DataSet::assign(s);
  data_ = s.data_;
  return *this;
}

Value const * StringSet::operator[](size_t n) const { 
  return ValueFactory::factory()->create(data_[n]);
}

bool StringSet::empty() const {
  return data_.empty();
}

size_t StringSet::size() const {
  return data_.size();
}

void StringSet::reserve(size_t N) {
  data_.reserve(N);
}

void StringSet::push_back(Value const & v) {
  data_.push_back(Access::get_string(v));
}

void StringSet::insert(size_t position, Value const & v) {
  data_.insert(data_.begin() + position, Access::get_string(v));
}

void StringSet::pop_back() {
  data_.pop_back();
}

void StringSet::erase(size_t position) {
  data_.erase(data_.begin() + position); 
}

void StringSet::erase(size_t first, size_t last) {
  data_.erase(data_.begin() + first, data_.begin() + last);
}

void StringSet::clear() {
  data_.clear();
}

Permutation StringSet::sorting_permutation(int dir = 1) const {
  return Permutation();
}

void StringSet::permute(Permutation const &) {
 
}

void StringSet::spew(ostream& out, char sep = ' ') const {

}

// Special accessors:
StringSet::representation const & StringSet::strings() const {
  return data_;
}

StringSet::representation & StringSet::strings() {
  return data_;
}
