// This is -*- C++ -*-
// $Id: Value.cpp,v 1.3 1998/12/30 22:35:34 hp Exp $

/* 
 * Value.cpp
 *
 * Copyright (C) 1998 Asger Alstrup Nielsen
 *
 * Developed by Asger Alstrup Nielsen <alstrup@diku.dk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include "Value.h"
#include "Access.h"
#include "DataType.h"

ValueFactory * ValueFactory::factory_ = 0;

// Implementation of the base Value class

void Value::set_value(string const & s) {
  string rest;
  Value * v = type()->parse(s, rest);
  assign(*v);
  if (v != 0) {
    delete v;
  }
}

string Value::format() const {
  return type()->format(*this);
}

void Value::set_type(DataType const & t) { 
  if (type_ != 0) delete type_;
  type_ = t.clone(); 
}

void Value::assign(Value const & v) { 
  set_type(*v.type_);
}

// Implementation of the Real value

Real::Real() : Value(RealType()) { 
  value_ = 0.0; 
}

Real::Real(double d) : Value(RealType()), value_(d) {
}

Real::Real(Value const & v) : Value(RealType()) { 
  assign(v);
}

void Real::set_real(double d) { 
  value_ = d; 
}

double Real::get_real() const { 
  return value_; 
}

void Real::set_value(string const & s) {
  string dummy;
  Value * t = type()->parse(s, dummy);
  Real::assign(*t);
  delete t;
}

string Real::format() const {
  return type()->format(*this);
};

void Real::assign(Value const & v) {
  Value::assign(v);
  value_ = Access::get_real(v);
}

// Implementation of the String value

void String::assign(Value const & v) {
  Value::assign(v);
  value_ = Access::get_string(v);
}
