// This is -*- C++ -*-
// $Id: WarningController.h,v 1.2 1998/12/14 18:48:28 alstrup Exp $

/* 
 * WarningController.h
 *
 * Copyright (C) 1998 Asger Alstrup Nielsen
 *
 * Developed by Asger Alstrup Nielsen <alstrup@diku.dk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_WARNINGCONTROLLER_H
#define _INC_WARNINGCONTROLLER_H

#include <string>
#include <iostream>

using namespace std;

/*
 * Abstract base class for warning controller.
 */
class WarningController {
public:
  WarningController(int warn = -1, int see_warns = -1)
    : warns(0), warnings_(warns), see_warnings_(see_warns)
  { }

  virtual ~WarningController() { }

  virtual void reset() {
    warns = 0;
  }

  // Present a warning
  virtual void warn(string const & message) = 0;

  // How many warnings do we tolerate?
  int warnings() const { return warnings_; }
  void set_warnings(int w) { warnings_ = w; }

  // How many warnings do we present?
  int see_warnings() const { return see_warnings_; }
  void set_see_warnings(int sw) { see_warnings_ = sw; }

protected:
  // Number of warnings/errors issued so far
  int warns;

private:
  // Number of warnings to tolerate
  int warnings_;

  // Number of warnings to present to the user
  int see_warnings_;
};

/*
 * Warnings reporting class.
 * This class handles the reporting of warnings.
 */
template<class _GUI>
class GUIWarningController : public WarningController {
public:
  GUIWarningController(_GUI & g, int warns = -1, int see_warns = -1)
    : WarningController(warns, see_warns), gui_(&g)
  { 
    reset();
  }

  // Present a warning to the user
  void warn(string const & message) {
    ++warns;

    if (see_warnings() == -1 || warns <= see_warnings()) {
     gui() << message << "\n";
      if (warns == see_warnings() && warnings() > see_warnings()) {
	gui() << "From now on, warnings are ignored.\n";
      }
    }

    if (warns > warnings() && warnings() != -1) {
      throw Exception("Too many warnings.");
    }
  }
private:
  _GUI & gui() { return *gui_; }

  // This is what we communicate with
  _GUI * gui_;
};

typedef GUIWarningController<ostream> StreamWarnings;

#endif
